@@ -10,7 +10,7 @@ use rand_distr::{Distribution, Normal, Uniform};
 fn simple_scatter_plot(show: bool) {
     let n: usize = 100;
     let t: Vec<f64> = linspace(0., 10., n).collect();
-    let y = t.iter().map(|x| x.sin()).collect();
+    let y: Vec<f64> = t.iter().map(|x| x.sin()).collect();
 
     let trace = Scatter::new(t, y).mode(Mode::Markers);
     let mut plot = Plot::new();
@@ -248,7 +248,7 @@ fn large_data_sets(show: bool) {
         .map(|args| args.0 * args.1.sin())
         .collect();
     let trace = Scatter::new(x, y)
-        .open_gl_mode(true)
+        .web_gl_mode(true)
         .mode(Mode::Markers)
         .marker(
             Marker::new()
@@ -473,7 +473,7 @@ fn filled_lines(show: bool) {
     .name("Premium")
     .show_legend(false);
     let trace3 = Scatter::new(
-        x1.clone(),
+        x1,
         vec![
             11.0, 9.0, 7.0, 5.0, 3.0, 1.0, 3.0, 5.0, 3.0, 1.0, -1.0, 1.0, 3.0, 1.0, -0.5, 1.0, 3.0,
             5.0, 7.0, 9.0,
@@ -496,12 +496,9 @@ fn filled_lines(show: bool) {
     )
     .line(Line::new().color(Rgb::new(0, 176, 246)))
     .name("Premium");
-    let trace6 = Scatter::new(
-        x2.clone(),
-        vec![10.0, 8.0, 6.0, 4.0, 2.0, 0.0, 2.0, 4.0, 2.0, 0.0],
-    )
-    .line(Line::new().color(Rgb::new(231, 107, 243)))
-    .name("Ideal");
+    let trace6 = Scatter::new(x2, vec![10.0, 8.0, 6.0, 4.0, 2.0, 0.0, 2.0, 4.0, 2.0, 0.0])
+        .line(Line::new().color(Rgb::new(231, 107, 243)))
+        .name("Ideal");
 
     let layout = Layout::new()
         .paper_background_color(Rgb::new(255, 255, 255))
