@@ -1,6 +1,3 @@
-#[cfg(feature = "orca")]
-extern crate plotly_orca;
-
 #[cfg(feature = "kaleido")]
 extern crate plotly_kaleido;
 
@@ -40,7 +37,14 @@ struct InlinePlotTemplate<'a> {
     plot_div_id: &'a str,
 }
 
-/// Image format for
+#[derive(Template)]
+#[template(path = "jupyter_notebook_plot.html", escape = "none")]
+struct JupyterNotebookPlotTemplate<'a> {
+    plot_data: &'a str,
+    plot_div_id: &'a str,
+}
+
+/// Image format for static image export.
 pub enum ImageFormat {
     PNG,
     JPEG,
@@ -50,6 +54,7 @@ pub enum ImageFormat {
     EPS,
 }
 
+
 /// A struct that implements `Trace` can be serialized to json format that is understood by Plotly.js.
 pub trait Trace {
     fn serialize(&self) -> String;
@@ -95,23 +100,34 @@ pub struct Plot {
 }
 
 const DEFAULT_HTML_APP_NOT_FOUND: &str = r#"Could not find default application for HTML files.
-Consider using the `to_html` method to save the plot instead. If using the `orca` feature the following
-additional formats are available accessed by following methods:
-- to_png
-- to_jpeg
-- to_webp
-- to_svg
-- to_pdf
-- to_eps
+Consider using the `to_html` method to save the plot instead. If using the `kaleido` feature the
+`save` method can be used to produce a static image in one of the following formats:
+- ImageFormat::PNG
+- ImageFormat::JPEG
+- ImageFormat::WEBP
+- ImageFormat::SVG
+- ImageFormat::PDF
+- ImageFormat::EPS
+
+used as follows:
+let plot = Plot::new();
+...
+let width = 1024;
+let height = 680;
+let scale = 1.0;
+plot.save("filename", ImageFormat::PNG, width, height, scale);
+
+See https://igiagkiozis.github.io/plotly/content/getting_started.html for further details.
 "#;
 
+
 impl Plot {
     /// Create a new `Plot`.
     pub fn new() -> Plot {
         Plot {
             traces: Vec::with_capacity(1),
-            layout: None,
             remote_plotly_js: true,
+            ..Default::default()
         }
     }
 
@@ -239,9 +255,10 @@ impl Plot {
     ///
     /// If `plot_div_id` is `None` the plot div id will be randomly generated, otherwise the user
     /// supplied div id is used.
-    pub fn to_inline_html(&self, plot_div_id: Option<&str>) -> String {
+    pub fn to_inline_html<T: Into<Option<&'static str>>>(&self, plot_div_id: T) -> String {
+        let plot_div_id = plot_div_id.into();
         match plot_div_id {
-            Some(id) => self.render_inline(id),
+            Some(id) => self.render_inline(id.as_ref()),
             None => {
                 let rand_id: String = thread_rng().sample_iter(&Alphanumeric).take(20).collect();
                 self.render_inline(rand_id.as_str())
@@ -249,6 +266,38 @@ impl Plot {
         }
     }
 
+    fn to_jupyter_notebook_html(&self) -> String {
+        let plot_div_id: String = thread_rng().sample_iter(&Alphanumeric).take(20).collect();
+        let plot_data = self.render_plot_data();
+
+        let tmpl = JupyterNotebookPlotTemplate {
+            plot_data: plot_data.as_str(),
+            plot_div_id: plot_div_id.as_str(),
+        };
+        tmpl.render().unwrap()
+    }
+
+    /// Display plot in Jupyter Notebook.
+    pub fn notebook_display(&self) {
+        let plot_data = self.to_jupyter_notebook_html();
+        println!("EVCXR_BEGIN_CONTENT text/html\n{}\nEVCXR_END_CONTENT", plot_data);
+    }
+
+    /// Display plot in Jupyter Lab.
+    pub fn lab_display(&self) {
+        let plot_data = self.to_json();
+        println!(
+            "EVCXR_BEGIN_CONTENT application/vnd.plotly.v1+json\n{}\nEVCXR_END_CONTENT",
+            plot_data
+        );
+    }
+
+    /// Displays the plot in Jupyter Lab; if running a Jupyter Notebook then use the
+    /// `notebook_display()` method instead.
+    pub fn evcxr_display(&self) {
+        self.lab_display();
+    }
+
     /// Saves the `Plot` to the selected image format.
     #[cfg(feature = "kaleido")]
     pub fn save<P: AsRef<Path>>(
@@ -278,79 +327,7 @@ impl Plot {
                 height,
                 scale,
             )
-            .expect(format!("failed to export plot to {:?}", filename.as_ref()).as_str());
-    }
-
-    /// Saves the `Plot` to png format.
-    #[cfg(feature = "orca")]
-    #[deprecated(
-        since = "0.5.0",
-        note = "Orca is no longer the recommended method to produce static images; please use the `kaleido` feature and the `Plot::save` method instead"
-    )]
-    pub fn to_png<P: AsRef<Path>>(&self, filename: P, width: usize, height: usize) {
-        let orca = plotly_orca::Orca::from(Plot::plotly_js_path());
-        let rendered = self.to_json();
-        orca.save_png(filename.as_ref(), &rendered, width, height);
-    }
-
-    /// Saves the `Plot` to jpeg format.
-    #[cfg(feature = "orca")]
-    #[deprecated(
-        since = "0.5.0",
-        note = "Orca is no longer the recommended method to produce static images; please use the `kaleido` feature and the `Plot::save` method instead"
-    )]
-    pub fn to_jpeg<P: AsRef<Path>>(&self, filename: P, width: usize, height: usize) {
-        let orca = plotly_orca::Orca::from(Plot::plotly_js_path());
-        let rendered = self.to_json();
-        orca.save_jpeg(filename.as_ref(), &rendered, width, height);
-    }
-
-    /// Saves the `Plot` to webp format.
-    #[cfg(feature = "orca")]
-    #[deprecated(
-        since = "0.5.0",
-        note = "Orca is no longer the recommended method to produce static images; please use the `kaleido` feature and the `Plot::save` method instead"
-    )]
-    pub fn to_webp<P: AsRef<Path>>(&self, filename: P, width: usize, height: usize) {
-        let orca = plotly_orca::Orca::from(Plot::plotly_js_path());
-        let rendered = self.to_json();
-        orca.save_webp(filename.as_ref(), &rendered, width, height);
-    }
-
-    /// Saves the `Plot` to svg format.
-    #[cfg(feature = "orca")]
-    #[deprecated(
-        since = "0.5.0",
-        note = "Orca is no longer the recommended method to produce static images; please use the `kaleido` feature and the `Plot::save` method instead"
-    )]
-    pub fn to_svg<P: AsRef<Path>>(&self, filename: P, width: usize, height: usize) {
-        let orca = plotly_orca::Orca::from(Plot::plotly_js_path());
-        let rendered = self.to_json();
-        orca.save_svg(filename.as_ref(), &rendered, width, height);
-    }
-
-    /// Saves the `Plot` to pdf format.
-    #[cfg(feature = "orca")]
-    #[deprecated(
-        since = "0.5.0",
-        note = "Orca is no longer the recommended method to produce static images; please use the `kaleido` feature and the `Plot::save` method instead"
-    )]
-    pub fn to_pdf<P: AsRef<Path>>(&self, filename: P, width: usize, height: usize) {
-        let orca = plotly_orca::Orca::from(Plot::plotly_js_path());
-        let rendered = self.to_json();
-        orca.save_pdf(filename.as_ref(), &rendered, width, height);
-    }
-
-    /// Saves the `Plot` to eps format.
-    #[cfg(feature = "orca")]
-    #[deprecated(
-        since = "0.5.0",
-        note = "Orca is no longer the recommended method to produce static images; please use the `kaleido` feature and the `Plot::save` method instead"
-    )]
-    pub fn to_eps<P: AsRef<Path>>(&self, filename: P, width: usize, height: usize) {
-        let orca = plotly_orca::Orca::from(Plot::plotly_js_path());
-        let rendered = self.to_json();
-        orca.save_eps(filename.as_ref(), &rendered, width, height);
+            .unwrap_or_else(|_| panic!("failed to export plot to {:?}", filename.as_ref()));
     }
 
     fn plotly_js_path() -> PathBuf {
@@ -418,7 +395,7 @@ impl Plot {
         tmpl.render().unwrap()
     }
 
-    fn to_json(&self) -> String {
+    pub fn to_json(&self) -> String {
         let mut plot_data: Vec<String> = Vec::new();
         for trace in self.traces.iter() {
             let s = trace.serialize();
@@ -484,16 +461,42 @@ mod tests {
         plot
     }
 
+    #[test]
+    fn test_to_json() {
+        let plot = create_test_plot();
+        let plot_json = plot.to_json();
+        println!("{}", plot_json);
+    }
+
     #[test]
     fn test_inline_plot() {
         let plot = create_test_plot();
-        let inline_plot_data = plot.to_inline_html(Some("replace_this_with_the_div_id"));
+        let inline_plot_data = plot.to_inline_html("replace_this_with_the_div_id");
         assert!(inline_plot_data.contains("replace_this_with_the_div_id"));
         println!("{}", inline_plot_data);
         let random_div_id = plot.to_inline_html(None);
         println!("{}", random_div_id);
     }
 
+    #[test]
+    fn test_jupyter_notebook_plot() {
+        let plot = create_test_plot();
+        let inline_plot_data = plot.to_jupyter_notebook_html();
+        println!("{}", inline_plot_data);
+    }
+
+    #[test]
+    fn test_notebook_display() {
+        let plot = create_test_plot();
+        plot.notebook_display();
+    }
+
+    #[test]
+    fn test_lab_display() {
+        let plot = create_test_plot();
+        plot.lab_display();
+    }
+
     #[test]
     #[cfg(feature = "kaleido")]
     fn test_save_to_png() {
@@ -560,72 +563,4 @@ mod tests {
         assert!(std::fs::remove_file(&dst).is_ok());
         assert!(!dst.exists());
     }
-
-    #[test]
-    #[cfg(feature = "orca")]
-    fn test_to_png() {
-        let plot = create_test_plot();
-        let dst = PathBuf::from("example.png");
-        plot.to_png(&dst, 1024, 680);
-        assert!(dst.exists());
-        assert!(std::fs::remove_file(&dst).is_ok());
-        assert!(!dst.exists());
-    }
-
-    #[test]
-    #[cfg(feature = "orca")]
-    fn test_to_jpeg() {
-        let plot = create_test_plot();
-        let dst = PathBuf::from("example.jpeg");
-        plot.to_jpeg(&dst, 1024, 680);
-        assert!(dst.exists());
-        assert!(std::fs::remove_file(&dst).is_ok());
-        assert!(!dst.exists());
-    }
-
-    #[test]
-    #[cfg(feature = "orca")]
-    fn test_to_webp() {
-        let plot = create_test_plot();
-        let dst = PathBuf::from("example.webp");
-        plot.to_webp(&dst, 1024, 680);
-        assert!(dst.exists());
-        assert!(std::fs::remove_file(&dst).is_ok());
-        assert!(!dst.exists());
-    }
-
-    #[test]
-    #[cfg(feature = "orca")]
-    fn test_to_svg() {
-        let plot = create_test_plot();
-        let dst = PathBuf::from("example.svg");
-        plot.to_svg(&dst, 1024, 680);
-        assert!(dst.exists());
-        assert!(std::fs::remove_file(&dst).is_ok());
-        assert!(!dst.exists());
-    }
-
-    #[test]
-    #[cfg(feature = "orca")]
-    fn test_to_pdf() {
-        let plot = create_test_plot();
-        let dst = PathBuf::from("example.pdf");
-        plot.to_pdf(&dst, 1024, 680);
-        assert!(dst.exists());
-        assert!(std::fs::remove_file(&dst).is_ok());
-        assert!(!dst.exists());
-    }
-
-    #[test]
-    #[ignore]
-    #[cfg(feature = "orca")]
-    #[cfg(any(target_os = "linux", target_os = "windows"))]
-    fn test_to_eps() {
-        let plot = create_test_plot();
-        let dst = PathBuf::from("example.eps");
-        plot.to_eps(&dst, 1024, 680);
-        assert!(dst.exists());
-        assert!(std::fs::remove_file(&dst).is_ok());
-        assert!(!dst.exists());
-    }
 }
